﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace P17_2
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class modCls : Microsoft.Xna.Framework.DrawableGameComponent
    {
        //Модель
        public Model myModel;
        //Мировая матрица, матрицы вида и проекции
        public Matrix WorldMatrix;
        public Matrix ViewMatrix;
        public Matrix ProjectMatrix;
        //Направление света
        public Vector3 LightDirection;
        //Матрица для отображения тени
        Matrix shadow;
        //Плоскость, на которой отображается тень
        Plane sPlane;
        //Соотношение сторон экрана
        public float aspectRatio;
        //Для управления графическим устройством
        GraphicsDeviceManager graphics;
        //Конструктор получает на вход 
        //игровой класс, модель, объект для управления графическим устройством и плоскость для вывода тени
        public modCls(Game game, Model mod, GraphicsDeviceManager grf, Plane pl)
            : base(game)
        {
            myModel = mod;
            graphics = grf;
            sPlane = pl;
            aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width /
(float)graphics.GraphicsDevice.Viewport.Height;
            LightDirection = new Vector3();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here
            base.Initialize();
        }

        public override void Draw(GameTime gameTime)
        {
            //Выводим объект
            foreach (ModelMesh mesh in myModel.Meshes)
            {
                //Для каждого эффекта в сети
                foreach (BasicEffect effect in mesh.Effects)
                {
                    //Включить источник направленного света №0
                    effect.DirectionalLight0.Enabled = true;
                    //Настроить параметры
                    effect.DirectionalLight0.DiffuseColor = Vector3.One;
                    effect.DirectionalLight0.SpecularColor = Vector3.One;
                    //Направление света - в класса Game1 мы меняем направление
                    //по клавиатурным командам
                    effect.DirectionalLight0.Direction = Vector3.Normalize(LightDirection);
                    //Включить освещение
                    effect.LightingEnabled = true;
                    //Установка матриц
                    effect.World = WorldMatrix;
                    effect.View = ViewMatrix;
                    effect.Projection = ProjectMatrix;
                }
                mesh.Draw();
            }

            //Создать матрицу тени
            shadow = Matrix.CreateShadow(-LightDirection, sPlane);
            //Очистить буфер трафаретов
            graphics.GraphicsDevice.Clear(ClearOptions.Stencil, Color.Black, 0, 0);
            //Включть буфер трафаретов
            graphics.GraphicsDevice.RenderState.StencilEnable = true;
            // Вывод на экран элемента буфера в том случае, если он установлен в 0         
            graphics.GraphicsDevice.RenderState.ReferenceStencil = 0;
            graphics.GraphicsDevice.RenderState.StencilFunction = CompareFunction.Equal;
            //При выводе применяем операцию увеличения
            graphics.GraphicsDevice.RenderState.StencilPass = StencilOperation.Increment;
            //Включаем альфа-смешивание для того, чтобы сделать тень полупрозрачной
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = true;
            graphics.GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
            graphics.GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
            //Выводим тень
            foreach (ModelMesh mesh in myModel.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.AmbientLightColor = Vector3.Zero;
                    effect.Alpha = 0.5f;
                    effect.DirectionalLight0.Enabled = false;
                    effect.DirectionalLight1.Enabled = false;
                    effect.DirectionalLight2.Enabled = false;
                    effect.View = ViewMatrix;
                    effect.Projection = ProjectMatrix;
                    //При выводе тени умножаем мировую матрицу
                    //на матрицу вывода тени
                    effect.World = WorldMatrix*shadow;
                }
                mesh.Draw();
            }
            //Отключить буфер
            graphics.GraphicsDevice.RenderState.StencilEnable = false;
            //Отключить альфа-смешивание
            graphics.GraphicsDevice.RenderState.AlphaBlendEnable = false;
            base.Draw(gameTime);
        }
    }
}
